-- ===========================================================================
-- Calc is a calculator that can add, subtract, multiply, and divide.
-- Copyright (C) 2016-2016 Eric T. Hughes (a.k.a. "Eboneezur")
--
--  This program is free software: you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program.  If not, see <http://www.gnu.org/licenses/>.
-- ===========================================================================
-- ===========================================================================
--- Class TPower
--
-- This class does little.  Not sure why I thought I needed to simulate a 
-- power button for a calculator.
-- ===========================================================================
do
  local nIsOnce = 0
  local nIsOn   = 1

-- -------------------------------------------------------------------------
--- Method ON
--
-- This method turns on the calculator
-- -------------------------------------------------------------------------
  local function ON()
    CalcFrameText:SetText("Calc ON")
    nIsOn = 1
    DEFAULT_CHAT_FRAME:AddMessage("[Calc] says: Power is turned on.", 0.0, 0.85, 0.0)

    if (nil ~= Broker_Calc.obj) then
      Broker_Calc.obj.icon = "Interface\\Addons\\Calc\\images\\bit_yes"
      Broker_Calc.obj.label = "Calc"
      Broker_Calc.obj.text = "ON"
    end
  end

-- -------------------------------------------------------------------------
--- Method OFF
--
-- This method turns off the calculator
-- -------------------------------------------------------------------------
  local function OFF()
    CalcFrameText:SetText("Calc OFF")
    nIsOn = 0
    DEFAULT_CHAT_FRAME:AddMessage("[Calc] says: Power is turned off.", 0.0, 0.85, 0.0)

    if (nil ~= Broker_Calc.obj) then
      Broker_Calc.obj.icon = "Interface\\Addons\\Calc\\images\\bit_default"
      Broker_Calc.obj.label = "Calc"
      Broker_Calc.obj.text = "OFF"
    end
  end
  
-- -------------------------------------------------------------------------
--- Method HIDE
--
-- This method hides the calculator
-- -------------------------------------------------------------------------
  local function HIDE()
    if (nil ~= Broker_Calc.obj) then
      Broker_Calc.obj.icon = "Interface\\Addons\\Calc\\images\\bit_default"
      Broker_Calc.obj.label = "Calc"
      Broker_Calc.obj.text = "Hide"
    end
  end

  
-- -------------------------------------------------------------------------
--- Method IsOn
--
-- This method returns 1 when the power is on or 0 when the power is off.
-- -------------------------------------------------------------------------
  local function IsOn()
    return nIsOn
  end

-- -------------------------------------------------------------------------
--- Method Toggle
--
-- This method determines whether to turn on or off the power.
-- @param self The given TPower object.
-- -------------------------------------------------------------------------
  local function Toggle(self)
    self.m_vecLocation.nIsInInstance, 
    self.m_vecLocation.strInstanceType = IsInInstance()

    if   (1      == self.m_vecLocation.nIsInInstance)   then
      if ("raid" == self.m_vecLocation.strInstanceType) then
        self.ON()
      else
        if (1 == nIsOn) then
          self.OFF()
        end
      end
    else
      if (1 == nIsOn) then
        CalcFrameText:SetText("Calc OFF")

        if   (1 == nIsOnce) then
        else
          DEFAULT_CHAT_FRAME:AddMessage(string.format("[Calc] says: Type /calc show, /calc hide, or /calc config."), 0.0, 0.85, 0.0)
          nIsOnce = 1
        end

        nIsOn = 0
        DEFAULT_CHAT_FRAME:AddMessage("[Calc] says: The calculator is hidden.", 0.0, 0.85, 0.0)
      end
    end

  end

-- -------------------------------------------------------------------------
--- Method Update
--
-- This method Updates the zone.
-- @param self The given TPower object.
-- -------------------------------------------------------------------------
  local function Update(self)
    self.m_vecLocation.strZone    = GetZoneText()
    self.m_vecLocation.strSubZone = GetSubZoneText()
    SetMapToCurrentZone()
  end


-- -------------------------------------------------------------------------
--- Method GetType
--
-- This method returns the type of this class.
-- -------------------------------------------------------------------------
  local function GetType()
      return "TPower"
   end

-- -------------------------------------------------------------------------
-- Method new_TPower
--
-- This method creates an object of this class.
-- -------------------------------------------------------------------------
  function new_TPower()

      local obj =
      {
        GetType = GetType,
        IsOn    = IsOn,
        OFF     = OFF,
        ON      = ON,
		HIDE    = HIDE,
        Toggle  = Toggle,
        Update  = Update,

        m_vecLocation =
        {
          strInstanceType = "",
          strZone         = "",
          strSubZone      = "",
          nX              = 0.0,
          nY              = 0.0,
          nIsInInstance   = 1,
        },
      }

      return obj
  end 
end


-- ===========================================================================
--- Class TSemaphore
--
-- This class defines a counting semaphore.
-- ===========================================================================
do
-- -------------------------------------------------------------------------
--- Method IsLocked
--
-- This method returns true if this counting semaphore has a lock.
-- @param self The given TSemaphore object.
-- -------------------------------------------------------------------------
  local function IsLocked(self)
    if 1 > self.nLockCount then
      return false
    else
      return true
    end
  end

-- -------------------------------------------------------------------------
--- Method IsUnLocked
--
-- This method returns true if this counting semaphore does not have a lock.
-- @param self The given TSemaphore object.
-- -------------------------------------------------------------------------
  local function IsUnLocked(self)
    if 0 < self.nLockCount then
      return false
    else
      return true
    end
  end

-- -------------------------------------------------------------------------
--- Method Lock
--
-- This method locks this counting semaphore and increases the lock count.
-- @param self The given TSemaphore object.
-- -------------------------------------------------------------------------
  local function Lock(self)
    self.nLockCount = self.nLockCount + 1
  end

-- -------------------------------------------------------------------------
--- Method UnLock
--
-- This method unlocks this counting semaphore and decreases the lock count.
-- @param self The given TSemaphore object.
-- -------------------------------------------------------------------------
  local function UnLock(self)
    if (0 < self.nLockCount) then
      self.nLockCount = self.nLockCount - 1
    end
  end

-- -------------------------------------------------------------------------
--- Method GetType
--
-- This method returns the type of this class.
-- -------------------------------------------------------------------------
  local function GetType()
    return "TSemaphore"
  end

-- -------------------------------------------------------------------------
--- Method new_TSemaphore
--
-- This method creates an object of this class.
-- -------------------------------------------------------------------------
  function new_TSemaphore()
    local obj =
    {
      GetType    = GetType,
      IsLocked   = IsLocked,
      IsUnLocked = IsUnLocked,
      Lock       = Lock,
      UnLock     = UnLock,
      nLockCount = 0,
    }

    return obj
  end 
end

  -- =========================================================================
  -- Class TStack
  --
  -- This class defines a stack.
  -- =========================================================================
  do
  -- -------------------------------------------------------------------------
  -- Method IsEmpty
  --
  -- This method returns true when the stack is empty or false otherwise.
  -- -------------------------------------------------------------------------
    local function IsEmpty(self)
      return (#self.m_vecItems == 0)
    end

  -- -------------------------------------------------------------------------
  -- Method Peek
  --
  -- This method returns the most recently stacked item without popping it off
  -- -------------------------------------------------------------------------
    local function Peek(self)
      local Item 
	  
      if (0 < #self.m_vecItems) then
        Item = self.m_vecItems[#self.m_vecItems]
      end
	  
      return Item
    end  
  
  -- -------------------------------------------------------------------------
  -- Method Pop
  --
  -- This method returns the most recently stacked item.
  -- -------------------------------------------------------------------------
    local function Pop(self)
      local Item 
	  
      if (0 < #self.m_vecItems) then
        Item = self.m_vecItems[#self.m_vecItems]
        table.remove(self.m_vecItems)
      end
	  
      return Item
    end  

  -- -------------------------------------------------------------------------
  -- Method Push
  --
  -- This method places an item on the stack.
  -- -------------------------------------------------------------------------
    local function Push(self, a_Item)
      table.insert(self.m_vecItems, a_Item)   
    end

  -- -------------------------------------------------------------------------
  -- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
     local function GetType()
       return "TStack"
     end

  -- -------------------------------------------------------------------------
  -- Method Size
  --
  -- This method returns the number of items on the stack.
  -- -------------------------------------------------------------------------
    local function Size(self)
      local numItemCount
	  
	  numItemCount = #self.m_vecItems
	  
      return numItemCount
    end  
	 
  -- -------------------------------------------------------------------------
  -- Method new_TStack
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TStack()
      local obj =
      {
        GetType    = GetType,
        IsEmpty    = IsEmpty,
		Peek       = Peek,
        Pop        = Pop,
        Push       = Push,
		Size       = Size,
        m_vecItems = {},
      }
	  
      return obj
    end
  end

do -- Class TCalculator
  -- -------------------------------------------------------------------------
  -- Method SetValue
  --
  -- This method pushes a value onto the stack for later evaluation.
  -- -------------------------------------------------------------------------
  local function SetValue(self)
    local strValue
    local numValue
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)
	numValue = tonumber(strValue)
    self.tblStack:Push(numValue)
	self.strValue = ""
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDecimalPoint
  --
  -- This method appends a decimal point to the string when there is not 
  -- already a decimal point in the string and it would not be the 15th
  -- character in the string.
  -- -------------------------------------------------------------------------
  local function AppendDecimalPoint(self)
    local strValue
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)

	-- Note: '.' is a wildcard character.  
	--       It should be escaped with a % to actually search for a decimal point.
	if (nil==string.find(strValue, '%.')) then
	  local numStringLength
	  
	  numStringLength = string.len(strValue)
	  
	  if (15 > numStringLength) then
	    strValue = string.format("%s.", strValue)
	  end
    end
	
	self.strValue = strValue
    return self.strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit0
  --
  -- This method appends a zero to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------
  local function AppendDigit0(self)
    local strValue
    local numStringLength	
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)
	numStringLength = string.len(strValue)
	
	if (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s0", strValue)
	  end
    end
	
	self.strValue = strValue
    return self.strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit1
  --
  -- This method appends a one to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------
  local function AppendDigit1(self)
    local strValue 
    local numStringLength
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numStringLength = string.len(strValue)
	
	if (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s1", strValue)
	  else
	    strValue = "1"
	  end
    end
	
	self.strValue = strValue
    return self.strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit2
  --
  -- This method appends a two to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------
  local function AppendDigit2(self)
    local strValue 
    local numStringLength
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numStringLength = string.len(strValue)
	
	if (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s2", strValue)
	  else
	    strValue = "2"
	  end
    end
	
	self.strValue = strValue
    return self.strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit3
  --
  -- This method appends a three to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------
  local function AppendDigit3(self)
    local strValue 
    local numStringLength
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numStringLength = string.len(strValue)
	
	if (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s3", strValue)
	  else
	    strValue = "3"
	  end
    end
	
	self.strValue = strValue
    return self.strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit4
  --
  -- This method appends a four to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------
  local function AppendDigit4(self)
    local strValue 
    local numStringLength
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numStringLength = string.len(strValue)
	
	if (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s4", strValue)
	  else
	    strValue = "4"
	  end
    end
	
	self.strValue = strValue
    return self.strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit5
  --
  -- This method appends a five to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------  
  local function AppendDigit5(self)
    local strValue 
    local numStringLength
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numStringLength = string.len(strValue)
	
	if (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s5", strValue)
	  else
	    strValue = "5"
	  end
    end
	
	self.strValue = strValue
    return self.strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit6
  --
  -- This method appends a six to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------  
  local function AppendDigit6(self)
    local strValue 
    local numStringLength
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numStringLength = string.len(strValue)
	
	if (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s6", strValue)
	  else
	    strValue = "6"
	  end
    end
	
	self.strValue = strValue
    return self.strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit7
  --
  -- This method appends a seven to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------  
  local function AppendDigit7(self)
    local strValue 
    local numStringLength
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numStringLength = string.len(strValue)
	
	if (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s7", strValue)
	  else
	    strValue = "7"
	  end
    end
	
	self.strValue = strValue
    return self.strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit8
  --
  -- This method appends an eight to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------
  local function AppendDigit8(self)
    local strValue 
    local numStringLength
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numStringLength = string.len(strValue)
	
	if (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s8", strValue)
	  else
	    strValue = "8"
	  end
    end
	
	self.strValue = strValue
    return self.strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit9
  --
  -- This method appends a two to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------
  local function AppendDigit9(self)
    local strValue 
    local numStringLength
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numStringLength = string.len(strValue)
	
	if (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s9", strValue)
	  else
	    strValue = "9"
	  end
    end
	
	self.strValue = strValue
    return self.strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method GetPlusSign
  --
  -- This method returns the plus sign.
  -- -------------------------------------------------------------------------
  local function GetPlusSign()
    return "+"
  end
  
  -- -------------------------------------------------------------------------
  -- Method GetMinusSign
  --
  -- This method returns the minus sign.
  -- -------------------------------------------------------------------------
  local function GetMinusSign()
    return "-"
  end
  
  -- -------------------------------------------------------------------------
  -- Method GetMultiplySign
  --
  -- This method returns the sign for multiplication.
  -- -------------------------------------------------------------------------
  local function GetMultiplySign()
    return "*"
  end
  
  -- -------------------------------------------------------------------------
  -- Method GetDivideSign
  --
  -- This method returns the sign for division.
  -- -------------------------------------------------------------------------
  local function GetDivideSign()
    return "/"
  end
  
  -- -------------------------------------------------------------------------
  -- Method Add
  --
  -- This method adds two numbers when there are two numbers to add.
  -- -------------------------------------------------------------------------
  local function Add(self)
    local numValue1
	local numValue2
	local numReturnValue
	local numStackSize
	
	self:SetValue()
	numStackSize = self.tblStack:Size()
	
	if (1 < numStackSize) then
	  numValue2 = self.tblStack:Pop()
	  numValue1 = self.tblStack:Pop()
	  numReturnValue = numValue1 + numValue2
	  self.tblStack:Push(numReturnValue)
	end
	
	self.strValue = ""
	return numReturnValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method Subtract
  --
  -- This method subtracts two numbers when there are two numbers to subtract.
  -- -------------------------------------------------------------------------
  local function Subtract(self)
    local numValue1
	local numValue2
	local numReturnValue
	local numStackSize
	
	self:SetValue()
	numStackSize = self.tblStack:Size()

	if (1 < numStackSize) then
	  numValue2 = self.tblStack:Pop()
	  numValue1 = self.tblStack:Pop()
	  numReturnValue = numValue1 - numValue2
	end
	
	self.strValue = ""
	return numReturnValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method Multiply
  --
  -- This method multiplies two numbers when there are two numbers to multiply.
  -- -------------------------------------------------------------------------
  local function Multiply(self)
    local numValue1
	local numValue2
	local numReturnValue
	local numStackSize
	
	self:SetValue()
	numStackSize = self.tblStack:Size()

	if (1 < numStackSize) then
      numValue2 = self.tblStack:Pop()
      numValue1 = self.tblStack:Pop()
      numReturnValue = numValue1 * numValue2
    end
	
	self.strValue = ""
	return numReturnValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method Divide
  --
  -- This method divides two numbers when there are two numbers to divide and
  -- when it is defined.
  -- -------------------------------------------------------------------------  
  local function Divide(self)
    local numValue1
	local numValue2
	local numReturnValue
	
	self:SetValue()
	numStackSize = self.tblStack:Size()

	if (1 < numStackSize) then
      numValue2 = self.tblStack:Pop()
	  
	  if (0 == numValue2) then -- protect against division by zero
	    self.tblStack:Push(numValue2)
      else
        numValue1 = self.tblStack:Pop()
        numReturnValue = numValue1 / numValue2
      end
	end
	
	self.strValue = ""
	return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method Reset
  --
  -- This method resets the calculator to its initial state.
  -- -------------------------------------------------------------------------
  local function Reset(self)
    local numItemCount
	
    self.strValue = ""
	
	numItemCount = self.tblStack:Size()
	
	if (0 < numItemCount) then
      for numIndex = 1, numItemCount, 1
      do
        self.tblStack:Pop()
      end
    end
  end
  
  -- -------------------------------------------------------------------------
  -- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
  local function GetType()
    return "TCalculator"
  end

  -- -------------------------------------------------------------------------
  -- Method new_TCalculator
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
  function new_TCalculator()
    local obj =
    {
      GetType            = GetType,
	  Reset              = Reset,
	  AppendDecimalPoint = AppendDecimalPoint,
	  AppendDigit0       = AppendDigit0,
	  AppendDigit1       = AppendDigit1,
	  AppendDigit2       = AppendDigit2,
	  AppendDigit3       = AppendDigit3,
	  AppendDigit4       = AppendDigit4,
	  AppendDigit5       = AppendDigit5,
	  AppendDigit6       = AppendDigit6,
	  AppendDigit7       = AppendDigit7,
	  AppendDigit8       = AppendDigit8,
	  AppendDigit9       = AppendDigit9,
	  GetPlusSign        = GetPlusSign,
	  GetMinusSign       = GetMinusSign,
	  GetMultiplySign    = GetMultiplySign,
	  GetDivideSign      = GetDivideSign,
	  SetValue           = SetValue,
	  Add                = Add,
	  Subtract           = Subtract,
	  Multiply           = Multiply,
	  Divide             = Divide,
	  tblStack           = new_TStack(),
	  strValue           = "",
    }
	  
    return obj
  end
end  

-- See: http://www.math.bas.bg/bantchev/place/rpn/rpn.lua.html
function Evaluate(a_strExpression)
 tblStack = {}  
 z = 0
 strText = ""
 
  for strToken in string.gmatch(a_strExpression, '%S+') do
    if string.find(strToken, '^[-+*/]$')  then
      if 2>#tblStack then z = nil break end
      y = table.remove(tblStack)
	  x = table.remove(tblStack)
      loadstring('z=x'..strToken..'y')()
    else
      z = tonumber(strToken)  if nil==z then break end
    end
    table.insert(tblStack, z)
  end
  
  numItemCount = #tblStack
  
  if 1==numItemCount and z then 
    strText = string.format("%s", z)
  elseif 1<numItemCount or nil==z then 
    strText = "error"
  end
  
  return strText
end

-- =========================================================================
-- Class TDialogCalculator
--
-- This class defines the calculator's user interface.
-- =========================================================================
do -- Class TDialogCalculator
  -- -------------------------------------------------------------------------
  --- Method SetDecimalPoint
  --
  -- This method appends a decimal point in the text box.
  -- -------------------------------------------------------------------------
  local function SetDecimalPoint()
    CalculatorDialog_TextBox:SetText(string.format("%s.", CalculatorDialog_TextBox:GetText()))
  end
  
  -- -------------------------------------------------------------------------
  --- Method SetText
  --
  -- This method adds the given text to the text box and resets the calculator.
  -- -------------------------------------------------------------------------
  local function SetText(a_strText)
    CalculatorDialog_TextBox:SetText(a_strText)
	self.tblCalculator:Reset()
  end
  
  -- -------------------------------------------------------------------------
  --- Method Reset
  --
  -- This method clears the text box.
  -- -------------------------------------------------------------------------
  local function Reset(self)
    CalculatorDialog_TextBox:SetText("")
  end
  
  -- -------------------------------------------------------------------------
  --- Method SetOperand
  --
  -- This method appends the given operand to the expression in the text box.
  -- -------------------------------------------------------------------------
  local function SetOperand(a_numOperand)
    CalculatorDialog_TextBox:SetText(string.format("%s%s ", CalculatorDialog_TextBox:GetText(), a_numOperand))
  end
  
  -- -------------------------------------------------------------------------
  --- Method SetOperator
  --
  -- This method evaluates the given operator and updates the text box.
  -- -------------------------------------------------------------------------
  local function SetOperator(self, a_strOperator)
    local numResult = nil
	local strPlusSign
	local strMinusSign
	local strMultiplySign
	local strDivideSign
	
	strPlusSign     = self.tblCalculator.GetPlusSign()
	strMinusSign    = self.tblCalculator.GetMinusSign()
	strMultiplySign = self.tblCalculator.GetMultiplySign()
	strDivideSign   = self.tblCalculator.GetDivideSign()
	
    if (strPlusSign == a_strOperator) then 
	  numResult = self.tblCalculator:Add()
	end
	
	if (strMinusSign == a_strOperator) then 
	  numResult = self.tblCalculator:Subtract() 
	end
	
	if (strMultiplySign == a_strOperator) then 
	  numResult = self.tblCalculator:Multiply() 
	end
	
	if (strDivideSign == a_strOperator) then 
	  numResult = self.tblCalculator:Divide()
	end
    
	if (nil~=numResult) then
      CalculatorDialog_TextBox:SetText(string.format("%s", numResult))
	end
  end

  do -- Class TDialogCalculator::TButtonZero
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent.SetText(self.tblParent.tblCalculator:AppendDigit0())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonZero"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonZero
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonZero()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do  -- Class TDialogCalculator::TButtonDecimalPoint
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent.SetText(self.tblParent.tblCalculator:AppendDecimalPoint())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonDecimalPoint"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonDecimalPoint
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonDecimalPoint()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonEqualSign
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
  --  local editbox
  --  local strText
	
  --  editbox=ChatEdit_ChooseBoxForSend(DEFAULT_CHAT_FRAME) --  Get an editbox
  --  ChatEdit_ActivateChat(editbox);--   Show the editbox
  --  strText = CalculatorDialog_TextBox:GetText()
  --  editbox:SetText(string.format("/simplecalc %s", strText)) -- Command goes here
  --  ChatEdit_OnEnterPressed(editbox);-- Process command and hide (runs ChatEdit_SendText() and ChatEdit_DeactivateChat() respectively)
  --  CalculatorDialog_TextBox:SetText("")
  --  strText = Evaluate(strText)
  -- CalculatorDialog_TextBox:SetText(strText)
     self.tblParent.tblCalculator:SetValue()	
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonEqualSign"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonEqualSign
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonEqualSign()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonPlusSign
  -- -----------------------------------numValue1-----------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator:GetPlusSign())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonPlusSign"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonPlusSign
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonPlusSign()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonOne
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent.SetText(self.tblParent.tblCalculator:AppendDigit1())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonOne"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonOne
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonOne()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonTwo
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent.SetText(self.tblParent.tblCalculator:AppendDigit2())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonTwo"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonTwo
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonTwo()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonThree
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent.SetText(self.tblParent.tblCalculator:AppendDigit3())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonThree"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonThree
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonThree()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonMinusSign
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator:GetMinusSign())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonMinusSign"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonMinusSign
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonMinusSign()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonFour
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent.SetText(self.tblParent.tblCalculator:AppendDigit4())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonFour"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonFour
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonFour()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonFive
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent.SetText(self.tblParent.tblCalculator:AppendDigit5())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonFive"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonFive
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonFive()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonSix
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent.SetText(self.tblParent.tblCalculator:AppendDigit6())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonSix"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonSix
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonSix()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonMultiplySign
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator:GetMultiplySign())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonMultiplySign"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonMultiplySign
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonMultiplySign()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonSeven
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent.SetText(self.tblParent.tblCalculator:AppendDigit7())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonSeven"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonSeven
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonSeven()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonEight
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent.SetText(self.tblParent.tblCalculator:AppendDigit8())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonEight"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonEight
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonEight()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- TDialogCalculator::TButtonNine
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent.SetText(self.tblParent.tblCalculator:AppendDigit9())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonNine"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonNine
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonNine()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonDivideSign
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:SetOperator(self.tblParent.tblCalculator.GetDivideSign())
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonDivideSign"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonDivideSign
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonDivideSign()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonMana
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  local numMana
	
	  numMana = UnitPowerMax('player')
	  self.tblParent.SetOperand(numMana)
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonMana"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonMana
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonMana()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonGold
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  local numGold
	
	  numGold = GetMoney() / 10000
	  self.tblParent.SetOperand(numGold)
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonGold"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonGold
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonGold()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonSilver
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  local numSilver
	
	  numSilver = GetMoney() / 100
	  self.tblParent.SetOperand(numSilver)
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonSilver"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonSilver
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonSilver()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonCopper
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  local numCopper
	
	  numCopper = GetMoney()
	  self.tblParent.SetOperand(numCopper)
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonCopper"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonCopper
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonCopper()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonHealth
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  local numHealth
	
	  numHealth = UnitHealthMax('player')
	  self.tblParent.SetOperand(numHealth)
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonHealth"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonHealth
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonHealth()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonAchieves
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  local numAchievementPoints
	
	  numAchievementPoints = GetTotalAchievementPoints()
	  self.tblParent.SetOperand(numAchievementPoints)
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonAchieves"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonAchieves
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonAchieves()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonConquest
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
      RequestPVPRewards()
    end
  
    local function OnClick(self)
	  local numConquest
	
	  numConquest = select(2,GetCurrencyInfo(390)) or 0	
	  self.tblParent.SetOperand(numConquest)
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonConquest"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonConquest
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonConquest()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonHonor
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  local numHonor
	
	  numHonor = select(2,GetCurrencyInfo(392)) or 0
	  self.tblParent.SetOperand(numHonor)
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonHonor"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonHonor
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonHonor()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonJustice
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  local numJustice
	
	  numJustice = select(2,GetCurrencyInfo(395)) or 0
	  self.tblParent.SetOperand(numJustice)
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonJustice"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonJustice
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonJustice()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonValor
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  local numValor
	
	  numValor = select(2,GetCurrencyInfo(396)) or 0
	  self.tblParent.SetOperand(numValor)
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonValor"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonValor
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonValor()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

      return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonVPCap
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  local numValorPointWeeklyCap
	
	  numValorPointWeeklyCap = select(5,GetCurrencyInfo(396))/100 - select(4,GetCurrencyInfo(396)) or 0
	  self.tblParent.SetOperand(numValorPointWeeklyCap)
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonVPCap"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonVPCap
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonVPCap()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

    return obj
    end 
  end

  do -- Class TDialogCalculator::TButtonReset
  -- -------------------------------------------------------------------------
  --- Method OnLoad
  --
  -- This method registers WoW API events.
  -- @param frame The given status button window.
  -- -------------------------------------------------------------------------
    local function OnLoad(frame)
      frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    end
  
    local function OnClick(self)
	  self.tblParent:Reset()
    end

  -- -------------------------------------------------------------------------
  --- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
    local function GetType()
      return "TButtonReset"
    end
	
  -- -------------------------------------------------------------------------
  --- Method new_TButtonReset
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TButtonReset()
      local obj =
      {
        GetType    = GetType,
	    OnLoad     = OnLoad,
	    OnClick    = OnClick,
		tblParent  = nil,
      }

    return obj
    end 
  end
  
-- -------------------------------------------------------------------------
--- Method GetType
--
-- This method returns the type of this class.
-- -------------------------------------------------------------------------
  local function GetType()
    return "TDialogCalculator"
  end
-- -------------------------------------------------------------------------
--- Method new_TDialogCalculator
--
-- This method creates an object of this class.
-- -------------------------------------------------------------------------
  function new_TDialogCalculator()
    local objDialogCalculator =
    {
      GetType               = GetType,
	  Reset                 = Reset,
	  SetDecimalPoint       = SetDecimalPoint,
	  SetText               = SetText,
	  SetOperand            = SetOperand,
	  SetOperator           = SetOperator,
      tblButtonZero         = new_TButtonZero(),
      tblButtonDecimalPoint = new_TButtonDecimalPoint(),
      tblButtonEqualSign    = new_TButtonEqualSign(),
      tblButtonPlusSign     = new_TButtonPlusSign(),
      tblButtonOne          = new_TButtonOne(),
      tblButtonTwo          = new_TButtonTwo(),
      tblButtonThree        = new_TButtonThree(),
      tblButtonMinusSign    = new_TButtonMinusSign(),
      tblButtonFour         = new_TButtonFour(),
      tblButtonFive         = new_TButtonFive(),
      tblButtonSix          = new_TButtonSix(),
      tblButtonMultiplySign = new_TButtonMultiplySign(),
      tblButtonSeven        = new_TButtonSeven(),
      tblButtonEight        = new_TButtonEight(),
      tblButtonNine         = new_TButtonNine(),
      tblButtonDivideSign   = new_TButtonDivideSign(),
      tblButtonMana         = new_TButtonMana(),
      tblButtonGold         = new_TButtonGold(),
      tblButtonSilver       = new_TButtonSilver(),
      tblButtonCopper       = new_TButtonCopper(),
      tblButtonHealth       = new_TButtonHealth(),
      tblButtonAchieves     = new_TButtonAchieves(),
      tblButtonConquest     = new_TButtonConquest(),
      tblButtonHonor        = new_TButtonHonor(),
      tblButtonJustice      = new_TButtonJustice(),
      tblButtonValor        = new_TButtonValor(),
      tblButtonVPCap        = new_TButtonVPCap(),
	  tblButtonReset        = new_TButtonReset(),
	  tblCalculator         = new_TCalculator(),
    }

    objDialogCalculator.tblButtonZero.tblParent = objDialogCalculator		
	objDialogCalculator.tblButtonDecimalPoint.tblParent = objDialogCalculator	
    objDialogCalculator.tblButtonEqualSign.tblParent = objDialogCalculator		
    objDialogCalculator.tblButtonPlusSign.tblParent = objDialogCalculator	
    objDialogCalculator.tblButtonOne.tblParent = objDialogCalculator	
    objDialogCalculator.tblButtonTwo.tblParent = objDialogCalculator		
    objDialogCalculator.tblButtonThree.tblParent = objDialogCalculator		
    objDialogCalculator.tblButtonMinusSign.tblParent = objDialogCalculator	
    objDialogCalculator.tblButtonFour.tblParent = objDialogCalculator		
    objDialogCalculator.tblButtonFive.tblParent = objDialogCalculator	
    objDialogCalculator.tblButtonSix.tblParent = objDialogCalculator		
    objDialogCalculator.tblButtonMultiplySign.tblParent = objDialogCalculator	
    objDialogCalculator.tblButtonSeven.tblParent = objDialogCalculator	
    objDialogCalculator.tblButtonEight.tblParent = objDialogCalculator	
    objDialogCalculator.tblButtonNine.tblParent = objDialogCalculator		
    objDialogCalculator.tblButtonDivideSign.tblParent = objDialogCalculator	
    objDialogCalculator.tblButtonMana.tblParent = objDialogCalculator	
    objDialogCalculator.tblButtonGold.tblParent = objDialogCalculator		
    objDialogCalculator.tblButtonSilver.tblParent = objDialogCalculator		
    objDialogCalculator.tblButtonCopper.tblParent = objDialogCalculator		
    objDialogCalculator.tblButtonHealth.tblParent = objDialogCalculator	
	objDialogCalculator.tblButtonAchieves.tblParent = objDialogCalculator	
    objDialogCalculator.tblButtonConquest.tblParent = objDialogCalculator		
	objDialogCalculator.tblButtonHonor.tblParent = objDialogCalculator	
	objDialogCalculator.tblButtonJustice.tblParent = objDialogCalculator	
	objDialogCalculator.tblButtonValor.tblParent = objDialogCalculator	
	objDialogCalculator.tblButtonVPCap.tblParent = objDialogCalculator	
	objDialogCalculator.tblButtonReset.tblParent = objDialogCalculator		

    return objDialogCalculator
  end 
end

tblDialogCalculator = new_TDialogCalculator()


-- ===========================================================================
--- Class TCalc
--
-- This class encapsulates the WoW API.
-- ===========================================================================
do
  -- Protect event handlers against WoW patches and programmer error.

  local semEventPLAYER_ENTERING_WORLD = new_TSemaphore()
  local nIsDisplayed = 0  -- Default to hidden

-- -------------------------------------------------------------------------
--- Method Hide
--
-- This method hides the calculator.
-- @param self The given TCalc object.
-- -------------------------------------------------------------------------
   local function Hide(self)
      CalcFrame:Hide()
	  CalculatorDialog:Hide()
      nIsDisplayed = 0
   end

-- -------------------------------------------------------------------------
--- Method IsDisplayed
--
-- This method returns 1 when the status button is shown or 
-- 0 when it is hidden.
-- @param self The given TCalc object.
-- -------------------------------------------------------------------------
   local function IsDisplayed(self)
     return nIsDisplayed
   end

-- -------------------------------------------------------------------------
--- Method Show
--
-- This method shows the calculator.
-- @param self The given TCalc object.
-- -------------------------------------------------------------------------
   local function Show(self)
      -- CalcFrame:Show()
	  CalculatorDialog:Show()
	  CalculatorDialog_TextBox:SetText("")
      nIsDisplayed = 1
   end

-- -------------------------------------------------------------------------
--- Method Toggle
--
-- This method shows/hides the calculator.
-- @param The given TCalc object.
-- -------------------------------------------------------------------------
  local function Toggle(self)
    if (0 == nIsDisplayed) then Show(self) else Hide(self) end
  end

-- -------------------------------------------------------------------------
--- Method OnLoad
--
-- This method registers WoW API events.
-- @param frame The given status button window.
-- -------------------------------------------------------------------------
  local function OnLoad(frame)
    frame:RegisterEvent("PLAYER_ENTERING_WORLD")
    frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    frame:RegisterForDrag("LeftButton")

    if (0 == nIsDisplayed) then Hide(self) else Show(self) end
  end


-- -------------------------------------------------------------------------
--- Method HandleEventPLAYER_ENTERING_WORLD
--
-- This method handles WoW API event "PLAYER_ENTERING_WORLD".
-- -------------------------------------------------------------------------
  local function HandleEventPLAYER_ENTERING_WORLD()
    if semEventPLAYER_ENTERING_WORLD:IsUnLocked() then
      semEventPLAYER_ENTERING_WORLD:Lock()

      tblPower:Toggle()

      semEventPLAYER_ENTERING_WORLD:UnLock()
    end
  end




-- -------------------------------------------------------------------------
--- Method OnEvent
--
-- This method handles WoW API events.
-- @param frame The given status button window.
-- @param event The given WoW event.
-- -------------------------------------------------------------------------
  local function OnEvent(frame, event, ...)
        if event == "PLAYER_ENTERING_WORLD" then HandleEventPLAYER_ENTERING_WORLD()
    end
  end    
  
    
-- -------------------------------------------------------------------------
--- Method GetType
--
-- This method returns the type of this class.
-- -------------------------------------------------------------------------
  local function GetType()
    return "TCalc"
  end

-- -------------------------------------------------------------------------
--- Method new_TCalc
--
-- This method creates an object of this class.
-- -------------------------------------------------------------------------
  function new_TCalc()
    local obj =
    {
        Hide            = Hide,
        IsDisplayed     = IsDisplayed,
        OnEvent         = OnEvent,
        OnLoad          = OnLoad,
        Show            = Show,
        Toggle          = Toggle,
    }

    obj.panel = CreateFrame( "Frame", "MyAddonPanel", UIParent )
    obj.panel.name = "Calc"

    local strTitle = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge")
    local strOverview = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine2 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine3 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine4 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine5 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine6 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine098 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine099 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine100 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine101 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine102 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine103 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine104 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine105 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine106 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine107 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strLine108 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strTableHeaderCol1 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormal")
    local strTableHeaderCol2 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormal")
    local strCell11 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell12 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell21 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell22 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell31 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell32 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell41 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell42 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell51 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell52 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell61 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
    local strCell62 = obj.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")

    strTitle:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -15)
    strOverview:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -45)
    strLine2:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -60)
    strLine3:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -75)
    strLine4:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -90)
    strLine5:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -105)
    strLine6:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -120)
    strTableHeaderCol1:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -150)
    strTableHeaderCol2:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 150, -150)
    strCell11:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -165)
    strCell21:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -180)
    strCell31:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -195)
    strCell41:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -210)
    strCell51:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -225)
    strCell61:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -240)
    strCell12:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 150, -165)
    strCell22:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 150, -180)
    strCell32:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 150, -195)
    strCell42:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 150, -210)
    strCell52:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 150, -225)
    strCell62:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 150, -240)
    strLine098:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -260)
    strLine099:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -275)
    strLine100:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -290)
    strLine101:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -305)
    strLine102:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -320)
    strLine103:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -335)
    strLine104:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -350)
    strLine105:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -365)
    strLine106:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -380)
    strLine107:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -395)
    strLine108:SetPoint("TOPLEFT", obj.panel, "TOPLEFT", 15, -410)

    strTitle:SetTextColor(0.0, 0.85, 0.0)
    strOverview:SetTextColor(1.0, 1.0, 1.0)
    strLine2:SetTextColor(1.0, 1.0, 1.0)
    strLine3:SetTextColor(1.0, 1.0, 1.0)
    strLine4:SetTextColor(1.0, 1.0, 1.0)
    strLine5:SetTextColor(1.0, 1.0, 1.0)
    strLine6:SetTextColor(1.0, 1.0, 1.0)
    strCell11:SetTextColor(1.0, 1.0, 1.0)
    strCell21:SetTextColor(1.0, 1.0, 1.0)
    strCell31:SetTextColor(1.0, 1.0, 1.0)
    strCell41:SetTextColor(1.0, 1.0, 1.0)
    strCell51:SetTextColor(1.0, 1.0, 1.0)
    strCell61:SetTextColor(1.0, 1.0, 1.0)
    strCell12:SetTextColor(1.0, 1.0, 1.0)
    strCell22:SetTextColor(1.0, 1.0, 1.0)
    strCell32:SetTextColor(1.0, 1.0, 1.0)
    strCell42:SetTextColor(1.0, 1.0, 1.0)
    strCell52:SetTextColor(1.0, 1.0, 1.0)
    strCell62:SetTextColor(1.0, 1.0, 1.0)
    strLine098:SetTextColor(1.0, 1.0, 1.0)
    strLine099:SetTextColor(1.0, 1.0, 1.0)
    strLine100:SetTextColor(1.0, 1.0, 1.0)
    strLine101:SetTextColor(1.0, 1.0, 1.0)
    strLine102:SetTextColor(1.0, 1.0, 1.0)
    strLine103:SetTextColor(1.0, 1.0, 1.0)
    strLine104:SetTextColor(1.0, 1.0, 1.0)
    strLine105:SetTextColor(1.0, 1.0, 1.0)
    strLine106:SetTextColor(1.0, 1.0, 1.0)
    strLine107:SetTextColor(1.0, 1.0, 1.0)
    strLine108:SetTextColor(1.0, 1.0, 1.0)
    strTitle:SetText("Calc V" ..GetAddOnMetadata("Calc", "Version" ) .. "   by " .. GetAddOnMetadata("Calc", "Author"))
    strOverview:SetText("Do you need to add, subtract, multiply, or divide?")
    strLine2:SetText   ("Calc is an rpn calculator add-on whose primary purpose is to do just that.")

    strTableHeaderCol1:SetText("Slash Command")
    strTableHeaderCol2:SetText("Description")
    strCell41:SetText("/calc show")
    strCell42:SetText("shows the calculator")             
    strCell51:SetText("/calc hide")
    strCell52:SetText("hides the calculator")             
    strCell61:SetText("/calc config")
    strCell62:SetText("display configuration options")             


    strLine098:SetText("Copyright (C) 2016-2016 Eric T. Hughes")
    strLine099:SetText("")
    strLine100:SetText("This add-on is free software: you can redistribute it and/or modify")
    strLine101:SetText("it under the terms of the GNU General Public License as published by")
    strLine102:SetText("the Free Software Foundation, either version 3 of the License, or")
    strLine103:SetText("(at your option) any later version.")
    strLine104:SetText("")
    strLine105:SetText("This add-on is distributed in the hope that it will be useful,")
    strLine106:SetText("but WITHOUT ANY WARRANTY; without even the implied warranty of")
    strLine107:SetText("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU")
    strLine108:SetText("General Public License for more details. <http://www.gnu.org/licenses/>")

    obj.panel.strTitle = strTitle
    obj.panel.strOverview = strOverview
    obj.panel.strLine2 = strLine2
    obj.panel.strTableHeader = strTableHeader
    obj.panel.strCell11 = strCell11
    obj.panel.strCell21 = strCell21
    obj.panel.strCell31 = strCell31
    obj.panel.strCell41 = strCell41
    obj.panel.strCell51 = strCell51
    obj.panel.strCell61 = strCell61
    obj.panel.strCell12 = strCell12
    obj.panel.strCell22 = strCell22
    obj.panel.strCell32 = strCell32
    obj.panel.strCell42 = strCell42    
    obj.panel.strCell52 = strCell52    
    obj.panel.strCell62 = strCell62    
    obj.panel.strLine098 = strLine098
    obj.panel.strLine099 = strLine099
    obj.panel.strLine100 = strLine100
    obj.panel.strLine101 = strLine101
    obj.panel.strLine102 = strLine102
    obj.panel.strLine103 = strLine103
    obj.panel.strLine104 = strLine104
    obj.panel.strLine105 = strLine105
    obj.panel.strLine106 = strLine106
    obj.panel.strLine107 = strLine107
    obj.panel.strLine108 = strLine108
    obj.panel:Hide()
    InterfaceOptions_AddCategory(obj.panel);
    return obj
  end 
end


tblPower = new_TPower()
tblCalc  = new_TCalc()



-- -------------------------------------------------------------------------
--- Function SlashCmdCalcHandler
--
-- This function does one of the following:
--    - ''       display slash command options
--    - 'config' display configuration options
-- @param val The given value.
-- -------------------------------------------------------------------------
function SlashCmdCalcHandler(val)
  if (nil ~= Broker_Calc) then
    Broker_Calc.obj.label = "Calc"
  end

      if val == "" or val == "?" or val == "help" or val == "HELP" or val == "Help" then
        strMessage = string.format("[Calc] says: Here are my slash commands:\n%3s\n%3s\n%3s", 
              "/calc show  - shows the calculator",
              "/calc hide   - hides the calculator", 
              "/calc config - display configuration options")
        DEFAULT_CHAT_FRAME:AddMessage(strMessage, 0.0, 0.85, 0.0)
		
        if (nil ~= Broker_Calc) then
          Broker_Calc.obj.text = ""
        end
      else
        if val == "config" or val == "CONFIG" or val == "Config" then
          if (nil ~= Broker_Calc) then
            Broker_Calc.obj.text = "CONFIG"
          end

          InterfaceOptionsFrame_OpenToCategory(tblCalc.panel)

        else
          if val == "show" or val == "SHOW" or val == "Show" then
            tblCalc:Show()
            if (nil ~= Broker_Calc) then
              Broker_Calc.obj.text = "SHOW"
            end
          else 
            if val == "hide" or val == "HIDE" or val == "Hide" then
              tblCalc:Hide()      
              if (nil ~= Broker_Calc) then
                Broker_Calc.obj.text = "HIDE"
              end
			else
              strMessage = string.format("[Calc] says: %s = %s", Evaluate(val), val)
              DEFAULT_CHAT_FRAME:AddMessage(strMessage, 0.0, 0.85, 0.0)
            end
          end
        end
      end
end 



SLASH_CALC1, SLASH_CALC2, SLASH_CALC3, SLASH_CALC4  = "/calc", "/calc show", "/calc hide", "/calc config"
SlashCmdList["CALC"] = SlashCmdCalcHandler